//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-pdf-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.Drawing;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Text;
using GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos.Basics
{
    // This sample shows how to hyphenate text that contains soft hyphen characters (0x00AD).
    // Breaks will be inserted at soft hyphen positions (if they are present in the text)
    // if TextLayout.WrapMode is set to WordWrap. Two properties are provided to control
    // hyphenation:
    // - TextLayout.SoftHyphenReplacementCharCode: specifies the character used as replacement
    //   for soft hyphen when breaking words across lines. By default this property is 0x002D
    //   (the Unicode hyphen-minus character). Setting this property to 0 breaks words without
    //   showing any visible hyphen character. Setting it to -1 prevents breaking words at soft
    //   hyphens).
    // - TextLayout.LinesBetweenConsecutiveHyphens: specifies the minimum number of non-
    //   hyphenated lines between lines ending in a hyphen. By default this property is zero.
    public class Hyphenation
    {
        public int CreatePDF(Stream stream)
        {
            // The online http://www.plazoo.com/en/tools.asp tool
            // was used to insert soft hyphens in the sample text from WordCharWrap:
            var str =
                "Lose noth&shy;ing in your doc&shy;u&shy;ments! Grape&shy;City Doc&shy;u&shy;ments for PDF " +
                "in&shy;cludes text and para&shy;graph format&shy;ting, spe&shy;cial char&shy;ac&shy;ters, " +
                "mul&shy;tiple lan&shy;guages, RTL sup&shy;port, ver&shy;tic&shy;al and ro&shy;tated text " +
                "on all sup&shy;por&shy;ted plat&shy;forms.";
            // Replace HTML soft hyphens with Unicode ones:
            str = str.Replace("&shy;", "\u00AD");

            var doc = new GcPdfDocument();
            var page = doc.NewPage();
            var g = page.Graphics;

            var tl = g.CreateTextLayout();
            tl.Append(str);
            tl.DefaultFormat.Font = StandardFonts.Times;
            tl.DefaultFormat.FontSize = 12;
            tl.MaxWidth = 72 * 3;

            // By default 0x002D (hyphen-minus) will be used as the hyphenation character
            // when breaking a line at a soft hyphen (0x00AD):
            tl.PerformLayout(true);

            var dy = tl.Lines[0].Height + 72 / 16;
            var rc = new RectangleF(72, 72 + dy, tl.MaxWidth.Value, 72 * 1.4F);

            g.DrawString("Default hyphenation:", tl.DefaultFormat, new PointF(rc.Left, rc.Top - dy));
            g.DrawTextLayout(tl, rc.Location);
            g.DrawRectangle(rc, Color.CornflowerBlue);

            rc.Offset(0, 72 * 2);
            // This will avoid hyphenating two consecutive lines:
            tl.LinesBetweenConsecutiveHyphens = 1;
            // Changing hyphenation options requires RecalculateGlyphs():
            tl.PerformLayout(true);
            g.DrawString("LinesBetweenConsecutiveHyphens: 1", tl.DefaultFormat, new PointF(rc.Left, rc.Top - dy));
            g.DrawTextLayout(tl, rc.Location);
            g.DrawRectangle(rc, Color.CornflowerBlue);

            rc.Offset(0, 72 * 2);
            // Reset previous setting:
            tl.LinesBetweenConsecutiveHyphens = 0;
            // Prevent hyphenating words at all:
            tl.SoftHyphenReplacementCharCode = -1;
            // Changing hyphenation options requires RecalculateGlyphs():
            tl.PerformLayout(true);
            g.DrawString("SoftHyphenReplacementCharCode: -1", tl.DefaultFormat, new PointF(rc.Left, rc.Top - dy));
            g.DrawTextLayout(tl, rc.Location);
            g.DrawRectangle(rc, Color.CornflowerBlue);

            // Done:
            doc.Save(stream);
            return doc.Pages.Count;
        }
    }
}
